@@ -75,8 +75,9 @@ int html##name##Push(htmlParserCtxtPtr ctxt, type value) {		\
 }									\
 type html##name##Pop(htmlParserCtxtPtr ctxt) {				\
     type ret;								\
-    if (ctxt->name##Nr <= 0) return(0);					\
+    if (ctxt->name##Nr < 0) return(0);					\
     ctxt->name##Nr--;							\
+    if (ctxt->name##Nr < 0) return(0);					\
     if (ctxt->name##Nr > 0)						\
 	ctxt->name = ctxt->name##Tab[ctxt->name##Nr - 1];		\
     else								\
@@ -87,6 +88,7 @@ type html##name##Pop(htmlParserCtxtPtr ctxt) {				\
 }									\
 
 PUSH_AND_POP(xmlNodePtr, node)
+PUSH_AND_POP(xmlChar*, name)
 
 /*
  * Macros for accessing the content. Those should be used only by the parser,
@@ -300,7 +302,7 @@ char *htmlStartClose[] = {
 "TITLE",	"P", NULL,
 "BODY",		"HEAD", "STYLE", "LINK", "TITLE", "P", NULL,
 "LI",		"P", "H1", "H2", "H3", "H4", "H5", "H6", "DL", "ADDRESS",
-		"PRE", "LISTING", "XMP", "HEAD", NULL,
+		"PRE", "LISTING", "XMP", "HEAD", "LI", NULL,
 "HR",		"P", "HEAD", NULL,
 "H1",		"P", "HEAD", NULL,
 "H2",		"P", "HEAD", NULL,
@@ -316,8 +318,10 @@ char *htmlStartClose[] = {
 "BLOCKQUOTE",	"P", "HEAD", NULL,
 "DL",		"P", "DT", "MENU", "DIR", "ADDRESS", "PRE", "LISTING",
 		"XMP", "HEAD", NULL,
-"DT",		"P", "MENU", "DIR", "ADDRESS", "PRE", "LISTING", "XMP", "HEAD", NULL,
-"DD",		"P", "MENU", "DIR", "ADDRESS", "PRE", "LISTING", "XMP", "HEAD", NULL,
+"DT",		"P", "MENU", "DIR", "ADDRESS", "PRE", "LISTING", "XMP",
+                "HEAD", "DD", NULL,
+"DD",		"P", "MENU", "DIR", "ADDRESS", "PRE", "LISTING", "XMP",
+                "HEAD", "DT", NULL,
 "UL",		"P", "HEAD", "OL", "MENU", "DIR", "ADDRESS", "PRE",
 		"LISTING", "XMP", NULL,
 "OL",		"P", "HEAD", "UL", NULL,
@@ -333,13 +337,13 @@ char *htmlStartClose[] = {
 "TABLE",	"P", "HEAD", "H1", "H2", "H3", "H4", "H5", "H6", "PRE",
 		"LISTING", "XMP", "A", NULL,
 "TH",		"TH", "TD", NULL,
-"TD",		"TH", "TD", NULL,
-"TR",		"TH", "TD", "TR", "CAPTION", "COL", "COLGROUP", NULL,
+"TD",		"TH", "TD", "P", NULL,
+"TR",		"TH", "TD", "TR", "CAPTION", "COL", "COLGROUP", "P", NULL,
 "THEAD",	"CAPTION", "COL", "COLGROUP", NULL,
 "TFOOT",	"TH", "TD", "TR", "CAPTION", "COL", "COLGROUP", "THEAD",
-		"TBODY", NULL,
+		"TBODY", "P", NULL,
 "TBODY",	"TH", "TD", "TR", "CAPTION", "COL", "COLGROUP", "THEAD",
-		"TFOOT", "TBODY", NULL,
+		"TFOOT", "TBODY", "P", NULL,
 "OPTGROUP",	"OPTION", NULL,
 "FIELDSET",	"LEGEND", "P", "HEAD", "H1", "H2", "H3", "H4", "H5", "H6",
 		"PRE", "LISTING", "XMP", "A", NULL,
@@ -443,14 +447,22 @@ htmlCheckAutoClose(const xmlChar *new, const xmlChar *old) {
  */
 void
 htmlAutoClose(htmlParserCtxtPtr ctxt, const xmlChar *new) {
-
-    while ((ctxt->node != NULL) && 
-           (htmlCheckAutoClose(new, ctxt->node->name))) {
+    xmlChar *oldname;
+    while ((ctxt->name != NULL) && 
+           (htmlCheckAutoClose(new, ctxt->name))) {
 #ifdef DEBUG
-	printf("htmlAutoClose: %s closes %s\n", new, ctxt->node->name);
+	fprintf(stderr,"htmlAutoClose: %s closes %s\n", new, ctxt->name);
 #endif
 	if ((ctxt->sax != NULL) && (ctxt->sax->endElement != NULL))
-	    ctxt->sax->endElement(ctxt->userData, ctxt->node->name);
+	    ctxt->sax->endElement(ctxt->userData, ctxt->name);
+	oldname = ctxt->name;
+	htmlnamePop(ctxt);
+	if (oldname != NULL) {
+#ifdef DEBUG
+	    fprintf(stderr,"htmlAutoClose: popped %s\n", oldname);
+#endif
+	    xmlFree(oldname);
+        }
     }
 }
 
@@ -464,18 +476,43 @@ htmlAutoClose(htmlParserCtxtPtr ctxt, const xmlChar *new) {
 void
 htmlAutoCloseOnClose(htmlParserCtxtPtr ctxt, const xmlChar *new) {
     htmlElemDescPtr info;
+    xmlChar *oldname;
+    int i;
 
-    while ((ctxt->node != NULL) && 
-           (xmlStrcmp(new, ctxt->node->name))) {
-	info = htmlTagLookup(ctxt->node->name);
+#ifdef DEBUG
+    fprintf(stderr,"Close of %s stack: %d elements\n", new, ctxt->nameNr);
+    for (i = 0;i < ctxt->nameNr;i++) 
+        fprintf(stderr,"%d : %s\n", i, ctxt->nameTab[i]);
+#endif
+
+    for (i = (ctxt->nameNr - 1);i >= 0;i--) {
+        if (!xmlStrcmp(new, ctxt->nameTab[i])) break;
+    }
+    if (i < 0) return;
+
+    while (xmlStrcmp(new, ctxt->name)) {
+	info = htmlTagLookup(ctxt->name);
 	if ((info == NULL) || (info->endTag == 1)) {
 #ifdef DEBUG
-	    printf("htmlAutoCloseOnClose: %s closes %s\n", new, ctxt->node->name);
+	    fprintf(stderr,"htmlAutoCloseOnClose: %s closes %s\n", new, ctxt->name);
 #endif
-	    if ((ctxt->sax != NULL) && (ctxt->sax->endElement != NULL))
-		ctxt->sax->endElement(ctxt->userData, ctxt->node->name);
-        } else
-	    break;
+        } else {
+	    if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
+		ctxt->sax->error(ctxt->userData,
+		 "Opening and ending tag mismatch: %s and %s\n",
+		                 new, ctxt->name);
+	    ctxt->wellFormed = 0;
+	}
+	if ((ctxt->sax != NULL) && (ctxt->sax->endElement != NULL))
+	    ctxt->sax->endElement(ctxt->userData, ctxt->name);
+	oldname = ctxt->name;
+	htmlnamePop(ctxt);
+	if (oldname != NULL) {
+#ifdef DEBUG
+	    fprintf(stderr,"htmlAutoCloseOnClose: popped %s\n", oldname);
+#endif
+	    xmlFree(oldname);
+	}	
     }
 }
 
@@ -799,7 +836,7 @@ htmlEntityLookup(const xmlChar *name) {
                     sizeof(html40EntitiesTable[0]));i++) {
         if (!xmlStrcmp(name, BAD_CAST html40EntitiesTable[i].name)) {
 #ifdef DEBUG
-            printf("Found entity %s\n", name);
+            fprintf(stderr,"Found entity %s\n", name);
 #endif
             return(&html40EntitiesTable[i]);
 	}
@@ -818,9 +855,7 @@ htmlEntityLookup(const xmlChar *name) {
  *
  * Subtitute the HTML entities by their value
  *
- * TODO: once the internal representation will be UTF-8, all entities
- *       will be substituable, in the meantime we only apply the substitution
- *       to the one with values in the 0-255 UNICODE range
+ * DEPRECATED !!!!
  *
  * Returns A newly allocated string with the substitution done. The caller
  *      must deallocate it !
@@ -1129,7 +1164,7 @@ htmlNewDoc(const xmlChar *URI, const xmlChar *ExternalID) {
     }
     memset(cur, 0, sizeof(xmlDoc));
 
-    cur->type = XML_DOCUMENT_NODE;
+    cur->type = XML_HTML_DOCUMENT_NODE;
     cur->version = NULL;
     cur->intSubset = NULL;
     xmlCreateIntSubset(cur, BAD_CAST "HTML", ExternalID, URI);
@@ -1239,28 +1274,29 @@ htmlParseName(htmlParserCtxtPtr ctxt) {
 /**
  * htmlParseHTMLAttribute:
  * @ctxt:  an HTML parser context
+ * @stop:  a char stop value
  * 
- * parse an HTML attribute value (without quotes).
+ * parse an HTML attribute value till the stop (quote), if
+ * stop is 0 then it stops at the first space
  *
- * Returns the Nmtoken parsed or NULL
+ * Returns the attribute parsed or NULL
  */
 
 xmlChar *
-htmlParseHTMLAttribute(htmlParserCtxtPtr ctxt) {
+htmlParseHTMLAttribute(htmlParserCtxtPtr ctxt, const xmlChar stop) {
     xmlChar buf[HTML_MAX_NAMELEN];
     int len = 0;
 
     GROW;
-    while ((!IS_BLANK(CUR)) && (CUR != '<') &&
-           (CUR != '&') && (CUR != '>') &&
-           (CUR != '\'') && (CUR != '"')) {
+    while ((CUR != 0) && (CUR != stop) && (CUR != '>')) {
+	if ((stop == 0) && (IS_BLANK(CUR))) break;
 	buf[len++] = CUR;
 	NEXT;
 	if (len >= HTML_MAX_NAMELEN) {
 	    fprintf(stderr, 
 	       "htmlParseHTMLAttribute: reached HTML_MAX_NAMELEN limit\n");
 	    while ((!IS_BLANK(CUR)) && (CUR != '<') &&
-		   (CUR != '&') && (CUR != '>') &&
+		   (CUR != '>') &&
 		   (CUR != '\'') && (CUR != '"'))
 		 NEXT;
 	    break;
@@ -1375,13 +1411,7 @@ htmlParseAttValue(htmlParserCtxtPtr ctxt) {
 
     if (CUR == '"') {
         NEXT;
-	ret = htmlDecodeEntities(ctxt, -1, '"', '<', 0);
-	if (CUR == '<') {
-	    if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
-		ctxt->sax->error(ctxt->userData,
-		   "Unescaped '<' not allowed in attributes values\n");
-	    ctxt->wellFormed = 0;
-	}
+	ret = htmlParseHTMLAttribute(ctxt, '"');
         if (CUR != '"') {
 	    if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
 		ctxt->sax->error(ctxt->userData, "AttValue: ' expected\n");
@@ -1390,13 +1420,7 @@ htmlParseAttValue(htmlParserCtxtPtr ctxt) {
 	    NEXT;
     } else if (CUR == '\'') {
         NEXT;
-	ret = htmlDecodeEntities(ctxt, -1, '\'', '<', 0);
-	if (CUR == '<') {
-	    if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
-		ctxt->sax->error(ctxt->userData,
-		   "Unescaped '<' not allowed in attributes values\n");
-	    ctxt->wellFormed = 0;
-	}
+	ret = htmlParseHTMLAttribute(ctxt, '\'');
         if (CUR != '\'') {
 	    if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
 		ctxt->sax->error(ctxt->userData, "AttValue: ' expected\n");
@@ -1407,14 +1431,13 @@ htmlParseAttValue(htmlParserCtxtPtr ctxt) {
         /*
 	 * That's an HTMLism, the attribute value may not be quoted
 	 */
-	ret = htmlParseHTMLAttribute(ctxt);
+	ret = htmlParseHTMLAttribute(ctxt, 0);
 	if (ret == NULL) {
 	    if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
 		ctxt->sax->error(ctxt->userData, "AttValue: no value found\n");
 	    ctxt->wellFormed = 0;
 	}
     }
-    
     return(ret);
 }
 
@@ -1666,12 +1689,6 @@ htmlParseComment(htmlParserCtxtPtr ctxt, int create) {
     while (IS_CHAR(CUR) &&
            ((CUR == ':') || (CUR != '>') ||
 	    (*r != '-') || (*q != '-'))) {
-	if ((*r == '-') && (*q == '-')) {
-	    if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
-	        ctxt->sax->error(ctxt->userData,
-	       "Comment must not contain '--' (double-hyphen)`\n");
-	    ctxt->wellFormed = 0;
-	}
         NEXT;r++;q++;
     }
     if (!IS_CHAR(CUR)) {
@@ -1903,10 +1920,9 @@ htmlParseAttribute(htmlParserCtxtPtr ctxt, xmlChar **value) {
  *
  * [NS 10] EmptyElement ::= '<' QName (S Attribute)* S? '/>'
  *
- * Returns the element name parsed
  */
 
-xmlChar *
+void
 htmlParseStartTag(htmlParserCtxtPtr ctxt) {
     xmlChar *name;
     xmlChar *attname;
@@ -1916,16 +1932,17 @@ htmlParseStartTag(htmlParserCtxtPtr ctxt) {
     int maxatts = 0;
     int i;
 
-    if (CUR != '<') return(NULL);
+    if (CUR != '<') return;
     NEXT;
 
+    GROW;
     name = htmlParseHTMLName(ctxt);
     if (name == NULL) {
 	if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
 	    ctxt->sax->error(ctxt->userData, 
 	     "htmlParseStartTag: invalid element name\n");
 	ctxt->wellFormed = 0;
-        return(NULL);
+        return;
     }
 
     /*
@@ -1943,7 +1960,9 @@ htmlParseStartTag(htmlParserCtxtPtr ctxt) {
            (CUR != '>') && 
 	   ((CUR != '/') || (NXT(1) != '>'))) {
 	const xmlChar *q = CUR_PTR;
+	int cons = ctxt->input->consumed;
 
+	GROW;
 	attname = htmlParseAttribute(ctxt, &attvalue);
         if ((attname != NULL) && (attvalue != NULL)) {
 	    /*
@@ -1952,12 +1971,13 @@ htmlParseStartTag(htmlParserCtxtPtr ctxt) {
 	    for (i = 0; i < nbatts;i += 2) {
 	        if (!xmlStrcmp(atts[i], attname)) {
 		    if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
-			ctxt->sax->error(ctxt->userData, "Attribute %s redefined\n",
-			                 name);
+			ctxt->sax->error(ctxt->userData,
+			                 "Attribute %s redefined\n",
+			                 attname);
 		    ctxt->wellFormed = 0;
 		    xmlFree(attname);
 		    xmlFree(attvalue);
-		    break;
+		    goto failed;
 		}
 	    }
 
@@ -1970,7 +1990,8 @@ htmlParseStartTag(htmlParserCtxtPtr ctxt) {
 		if (atts == NULL) {
 		    fprintf(stderr, "malloc of %ld byte failed\n",
 			    maxatts * (long)sizeof(xmlChar *));
-		    return(NULL);
+		    if (name != NULL) xmlFree(name);
+		    return;
 		}
 	    } else if (nbatts + 2 < maxatts) {
 	        maxatts *= 2;
@@ -1978,7 +1999,8 @@ htmlParseStartTag(htmlParserCtxtPtr ctxt) {
 		if (atts == NULL) {
 		    fprintf(stderr, "realloc of %ld byte failed\n",
 			    maxatts * (long)sizeof(xmlChar *));
-		    return(NULL);
+		    if (name != NULL) xmlFree(name);
+		    return;
 		}
 	    }
 	    atts[nbatts++] = attname;
@@ -1987,8 +2009,9 @@ htmlParseStartTag(htmlParserCtxtPtr ctxt) {
 	    atts[nbatts + 1] = NULL;
 	}
 
+failed:
 	SKIP_BLANKS;
-        if (q == CUR_PTR) {
+        if ((cons == ctxt->input->consumed) && (q == CUR_PTR)) {
 	    if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
 	        ctxt->sax->error(ctxt->userData, 
 	         "htmlParseStartTag: problem parsing attributes\n");
@@ -2000,6 +2023,10 @@ htmlParseStartTag(htmlParserCtxtPtr ctxt) {
     /*
      * SAX: Start of Element !
      */
+    htmlnamePush(ctxt, xmlStrdup(name));
+#ifdef DEBUG
+    fprintf(stderr,"Start of element %s: pushed %s\n", name, ctxt->name);
+#endif    
     if ((ctxt->sax != NULL) && (ctxt->sax->startElement != NULL))
         ctxt->sax->startElement(ctxt->userData, name, atts);
 
@@ -2007,13 +2034,12 @@ htmlParseStartTag(htmlParserCtxtPtr ctxt) {
         for (i = 0;i < nbatts;i++) xmlFree((xmlChar *) atts[i]);
 	xmlFree(atts);
     }
-    return(name);
+    if (name != NULL) xmlFree(name);
 }
 
 /**
  * htmlParseEndTag:
  * @ctxt:  an HTML parser context
- * @tagname:  the tag name as parsed in the opening tag.
  *
  * parse an end of tag
  *
@@ -2025,8 +2051,9 @@ htmlParseStartTag(htmlParserCtxtPtr ctxt) {
  */
 
 void
-htmlParseEndTag(htmlParserCtxtPtr ctxt, const xmlChar *tagname) {
+htmlParseEndTag(htmlParserCtxtPtr ctxt) {
     xmlChar *name;
+    xmlChar *oldname;
     int i;
 
     if ((CUR != '<') || (NXT(1) != '/')) {
@@ -2051,27 +2078,26 @@ htmlParseEndTag(htmlParserCtxtPtr ctxt, const xmlChar *tagname) {
 	NEXT;
 
     /*
-     * Check that we are not closing an already closed tag,
-     * <p><b>...</p></b> is a really common error !
+     * If the name read is not one of the element in the parsing stack
+     * then return, it's just an error.
      */
-    for (i = ctxt->nodeNr - 1;i >= 0;i--) {
-        if ((ctxt->nodeTab[i] != NULL) &&
-	    (!xmlStrcmp(tagname, ctxt->nodeTab[i]->name)))
-	    break;
+    for (i = (ctxt->nameNr - 1);i >= 0;i--) {
+        if (!xmlStrcmp(name, ctxt->nameTab[i])) break;
     }
     if (i < 0) {
 	if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
-	    ctxt->sax->error(ctxt->userData, 
-	                     "htmlParseEndTag: unexpected close for tag %s\n",
-			     tagname);
+	    ctxt->sax->error(ctxt->userData,
+	     "Unexpected end tag : %s\n", name);
 	xmlFree(name);
 	ctxt->wellFormed = 0;
 	return;
     }
 
+
     /*
      * Check for auto-closure of HTML elements.
      */
+
     htmlAutoCloseOnClose(ctxt, name);
 
     /*
@@ -2079,13 +2105,16 @@ htmlParseEndTag(htmlParserCtxtPtr ctxt, const xmlChar *tagname) {
      * With the exception that the autoclose may have popped stuff out
      * of the stack.
      */
-    if (xmlStrcmp(name, tagname)) {
-        if ((ctxt->node != NULL) && 
-	    (xmlStrcmp(ctxt->node->name, name))) {
+    if (xmlStrcmp(name, ctxt->name)) {
+#ifdef DEBUG
+	fprintf(stderr,"End of tag %s: expecting %s\n", name, ctxt->name);
+#endif
+        if ((ctxt->name != NULL) && 
+	    (xmlStrcmp(ctxt->name, name))) {
 	    if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
 		ctxt->sax->error(ctxt->userData,
 		 "Opening and ending tag mismatch: %s and %s\n",
-		                 name, ctxt->node->name);
+		                 name, ctxt->name);
 	    ctxt->wellFormed = 0;
         }
     }
@@ -2093,8 +2122,22 @@ htmlParseEndTag(htmlParserCtxtPtr ctxt, const xmlChar *tagname) {
     /*
      * SAX: End of Tag
      */
-    if ((ctxt->sax != NULL) && (ctxt->sax->endElement != NULL))
-        ctxt->sax->endElement(ctxt->userData, name);
+    oldname = ctxt->name;
+    if (!xmlStrcmp(oldname, name)) {
+	if ((ctxt->sax != NULL) && (ctxt->sax->endElement != NULL))
+	    ctxt->sax->endElement(ctxt->userData, name);
+	htmlnamePop(ctxt);
+	if (oldname != NULL) {
+#ifdef DEBUG
+	    fprintf(stderr,"End of tag %s: popping out %s\n", name, oldname);
+#endif
+	    xmlFree(oldname);
+#ifdef DEBUG
+	} else {
+	    fprintf(stderr,"End of tag %s: stack empty !!!\n", name);
+#endif
+	}
+    }
 
     if (name != NULL)
 	xmlFree(name);
@@ -2156,18 +2199,30 @@ htmlParseReference(htmlParserCtxtPtr ctxt) {
  */
 
 void
-htmlParseContent(htmlParserCtxtPtr ctxt, const xmlChar *name) {
-    htmlNodePtr currentNode;
+htmlParseContent(htmlParserCtxtPtr ctxt) {
+    xmlChar *currentNode;
+    int depth;
 
-    currentNode = ctxt->node;
-    while ((CUR != '<') || (NXT(1) != '/')) {
+    currentNode = ctxt->name;
+    depth = ctxt->nameNr;
+    while (1) {
 	const xmlChar *test = CUR_PTR;
 
+        GROW;
+	/*
+	 * Our tag or one of it's parent or children is ending.
+	 */
+        if ((CUR == '<') && (NXT(1) == '/')) {
+	    htmlParseEndTag(ctxt);
+	    return;
+        }
+
 	/*
 	 * Has this node been popped out during parsing of
 	 * the next element
 	 */
-        if (currentNode != ctxt->node) return;
+        if ((currentNode != ctxt->name) &&
+	    (depth >= ctxt->nameNr)) return;
 
 	/*
 	 * First case :  a comment
@@ -2200,19 +2255,17 @@ htmlParseContent(htmlParserCtxtPtr ctxt, const xmlChar *name) {
 	}
 
 	if (test == CUR_PTR) {
-	    if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
-	        ctxt->sax->error(ctxt->userData,
-		     "detected an error in element content\n");
-	    ctxt->wellFormed = 0;
+	    if (ctxt->node != NULL) {
+		if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
+		    ctxt->sax->error(ctxt->userData,
+			 "detected an error in element content\n");
+		ctxt->wellFormed = 0;
+	    }
             break;
 	}
+
         GROW;
     }
-
-    /*
-     * parse the end of tag: '</' should be here.
-     */
-    htmlParseEndTag(ctxt, name);
 }
 
 /**
@@ -2229,10 +2282,12 @@ htmlParseContent(htmlParserCtxtPtr ctxt, const xmlChar *name) {
 void
 htmlParseElement(htmlParserCtxtPtr ctxt) {
     const xmlChar *openTag = CUR_PTR;
+    xmlChar *oldname;
     xmlChar *name;
-    htmlNodePtr currentNode;
+    xmlChar *currentNode = NULL;
     htmlElemDescPtr info;
     htmlParserNodeInfo node_info;
+    int depth = ctxt->nameNr;
 
     /* Capture start position */
     if (ctxt->record_info) {
@@ -2241,11 +2296,23 @@ htmlParseElement(htmlParserCtxtPtr ctxt) {
 	node_info.begin_line = ctxt->input->line;
     }
 
-    name = htmlParseStartTag(ctxt);
-    if (name == NULL) {
+    oldname = ctxt->name;
+    htmlParseStartTag(ctxt);
+    name = ctxt->name;
+#ifdef DEBUG
+    if (oldname == NULL)
+	fprintf(stderr, "Start of element %s\n", name);
+    else if (name == NULL)	
+	fprintf(stderr, "Start of element failed, was %s\n", oldname);
+    else	
+	fprintf(stderr, "Start of element %s, was %s\n", name, oldname);
+#endif
+    if (((depth == ctxt->nameNr) && (oldname == ctxt->name)) ||
+        (name == NULL)) {
+	if (CUR == '>')
+	    NEXT;
         return;
     }
-    currentNode = ctxt->node;
 
     /*
      * Lookup the info for that element.
@@ -2271,7 +2338,13 @@ htmlParseElement(htmlParserCtxtPtr ctxt) {
         SKIP(2);
 	if ((ctxt->sax != NULL) && (ctxt->sax->endElement != NULL))
 	    ctxt->sax->endElement(ctxt->userData, name);
-	xmlFree(name);
+	oldname = ctxt->name;
+#ifdef DEBUG
+        fprintf(stderr,"End of tag the XML way: popping out %s\n", oldname);
+#endif
+	htmlnamePop(ctxt);
+	if (oldname != NULL)
+	    xmlFree(oldname);
 	return;
     }
 
@@ -2286,8 +2359,17 @@ htmlParseElement(htmlParserCtxtPtr ctxt) {
 	/*
 	 * end of parsing of this node.
 	 */
-	nodePop(ctxt);
-	xmlFree(name);
+	if (!xmlStrcmp(name, ctxt->name)) { 
+	    nodePop(ctxt);
+	    xmlFree(name);
+	    oldname = ctxt->name;
+#ifdef DEBUG
+	    fprintf(stderr,"End of start tag problem: popping out %s\n", oldname);
+#endif
+	    htmlnamePop(ctxt);
+	    if (oldname != NULL)
+		xmlFree(oldname);
+	}    
 
 	/*
 	 * Capture end position and add node
@@ -2296,7 +2378,7 @@ htmlParseElement(htmlParserCtxtPtr ctxt) {
 	   node_info.end_pos = ctxt->input->consumed +
 			      (CUR_PTR - ctxt->input->base);
 	   node_info.end_line = ctxt->input->line;
-	   node_info.node = currentNode;
+	   node_info.node = ctxt->node;
 	   xmlParserAddNodeInfo(ctxt, &node_info);
 	}
 	return;
@@ -2308,41 +2390,46 @@ htmlParseElement(htmlParserCtxtPtr ctxt) {
     if ((info != NULL) && (info->empty)) {
 	if ((ctxt->sax != NULL) && (ctxt->sax->endElement != NULL))
 	    ctxt->sax->endElement(ctxt->userData, name);
-	xmlFree(name);
+	oldname = ctxt->name;
+#ifdef DEBUG
+	fprintf(stderr,"End of empty tag %s : popping out %s\n", name, oldname);
+#endif
+	htmlnamePop(ctxt);
+	if (oldname != NULL)
+	    xmlFree(oldname);
 	return;
     }
 
     /*
      * Parse the content of the element:
      */
-    currentNode = ctxt->node;
-    htmlParseContent(ctxt, name);
-
-    /*
-     * check whether the element get popped due to auto closure
-     * on start tag
-     */
-    if (currentNode != ctxt->node) {
-	xmlFree(name);
-        return;
-    }
+    currentNode = ctxt->name;
+    depth = ctxt->nameNr;
+    while (IS_CHAR(CUR)) {
+	htmlParseContent(ctxt);
+	if (ctxt->nameNr < depth) break; 
+    }	
 
     if (!IS_CHAR(CUR)) {
 	if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
 	    ctxt->sax->error(ctxt->userData,
-	         "Premature end of data in tag %.30s\n", openTag);
+	         "Premature end of data in tag %s\n", currentNode);
 	ctxt->wellFormed = 0;
 
 	/*
 	 * end of parsing of this node.
 	 */
 	nodePop(ctxt);
-	xmlFree(name);
+	oldname = ctxt->name;
+#ifdef DEBUG
+	fprintf(stderr,"Premature end of tag %s : popping out %s\n", name, oldname);
+#endif
+	htmlnamePop(ctxt);
+	if (oldname != NULL)
+	    xmlFree(oldname);
 	return;
     }
 
-    xmlFree(name);
-
     /*
      * Capture end position and add node
      */
@@ -2350,7 +2437,7 @@ htmlParseElement(htmlParserCtxtPtr ctxt) {
        node_info.end_pos = ctxt->input->consumed +
                           (CUR_PTR - ctxt->input->base);
        node_info.end_line = ctxt->input->line;
-       node_info.node = currentNode;
+       node_info.node = ctxt->node;
        xmlParserAddNodeInfo(ctxt, &node_info);
     }
 }
@@ -2381,20 +2468,23 @@ htmlParseDocument(htmlParserCtxtPtr ctxt) {
     /*
      * Wipe out everything which is before the first '<'
      */
-    if (IS_BLANK(CUR)) {
-	if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
-	    ctxt->sax->error(ctxt->userData,
-	    "Extra spaces at the beginning of the document are not allowed\n");
-	ctxt->wellFormed = 0;
-	SKIP_BLANKS;
-    }
-
+    SKIP_BLANKS;
     if (CUR == 0) {
 	if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
 	    ctxt->sax->error(ctxt->userData, "Document is empty\n");
 	ctxt->wellFormed = 0;
     }
 
+    /*
+     * Parse possible comments before any content
+     */
+    while ((CUR == '<') && (NXT(1) == '!') &&
+           (NXT(2) == '-') && (NXT(3) == '-')) {
+        ctxt->myDoc = htmlNewDoc(NULL, NULL);
+        htmlParseComment(ctxt, 1);	   
+	SKIP_BLANKS;
+    }	   
+
 
     /*
      * Then possibly doc type declaration(s) and more Misc
@@ -2419,7 +2509,7 @@ htmlParseDocument(htmlParserCtxtPtr ctxt) {
     /*
      * Time to start parsing the tree itself
      */
-    htmlParseElement(ctxt);
+    htmlParseContent(ctxt);
 
     /*
      * SAX: end of the document processing.
@@ -2449,13 +2539,21 @@ htmlInitParserCtxt(htmlParserCtxtPtr ctxt)
 {
     htmlSAXHandler *sax;
 
+    if (ctxt == NULL) return;
+    memset(ctxt, 0, sizeof(htmlParserCtxt));
+
     sax = (htmlSAXHandler *) xmlMalloc(sizeof(htmlSAXHandler));
     if (sax == NULL) {
         fprintf(stderr, "htmlInitParserCtxt: out of memory\n");
     }
+    memset(sax, 0, sizeof(htmlSAXHandler));
 
     /* Allocate the Input stack */
-    ctxt->inputTab = (htmlParserInputPtr *) xmlMalloc(5 * sizeof(htmlParserInputPtr));
+    ctxt->inputTab = (htmlParserInputPtr *) 
+                      xmlMalloc(5 * sizeof(htmlParserInputPtr));
+    if (ctxt->inputTab == NULL) {
+        fprintf(stderr, "htmlInitParserCtxt: out of memory\n");
+    }
     ctxt->inputNr = 0;
     ctxt->inputMax = 5;
     ctxt->input = NULL;
@@ -2469,6 +2567,12 @@ htmlInitParserCtxt(htmlParserCtxtPtr ctxt)
     ctxt->nodeMax = 10;
     ctxt->node = NULL;
 
+    /* Allocate the Name stack */
+    ctxt->nameTab = (xmlChar **) xmlMalloc(10 * sizeof(xmlChar *));
+    ctxt->nameNr = 0;
+    ctxt->nameMax = 10;
+    ctxt->name = NULL;
+
     if (sax == NULL) ctxt->sax = &htmlDefaultSAXHandler;
     else {
         ctxt->sax = sax;
@@ -2480,6 +2584,7 @@ htmlInitParserCtxt(htmlParserCtxtPtr ctxt)
     ctxt->replaceEntities = 0;
     ctxt->html = 1;
     ctxt->record_info = 0;
+    ctxt->validate = 0;
     xmlInitNodeInfoSeq(&ctxt->node_seq);
 }
 
@@ -2495,6 +2600,7 @@ void
 htmlFreeParserCtxt(htmlParserCtxtPtr ctxt)
 {
     htmlParserInputPtr input;
+    xmlChar *oldname;
 
     if (ctxt == NULL) return;
 
@@ -2503,6 +2609,12 @@ htmlFreeParserCtxt(htmlParserCtxtPtr ctxt)
     }
 
     if (ctxt->nodeTab != NULL) xmlFree(ctxt->nodeTab);
+    while ((oldname = ctxt->name) != NULL) {
+        htmlnamePop(ctxt);
+	if (oldname != NULL)
+	    xmlFree(oldname);
+    }
+    if (ctxt->nameTab != NULL) xmlFree(ctxt->nameTab);
     if (ctxt->inputTab != NULL) xmlFree(ctxt->inputTab);
     if (ctxt->version != NULL) xmlFree((char *) ctxt->version);
     if ((ctxt->sax != NULL) && (ctxt->sax != &htmlDefaultSAXHandler))
@@ -2537,22 +2649,12 @@ htmlCreateDocParserCtxt(xmlChar *cur, const char *encoding) {
 	xmlFree(ctxt);
 	return(NULL);
     }
+    memset(input, 0, sizeof(htmlParserInput));
 
-    /*
-     * plug some encoding conversion routines here. !!!
-    if (encoding != NULL) {
-	enc = htmlDetectCharEncoding(cur);
-	htmlSwitchEncoding(ctxt, enc);
-    }
-     */
-
-    input->filename = NULL;
     input->line = 1;
     input->col = 1;
     input->base = cur;
     input->cur = cur;
-    input->free = NULL;
-    input->buf = NULL;
 
     inputPush(ctxt, input);
     return(ctxt);
@@ -2647,6 +2749,7 @@ htmlCreateFileParserCtxt(const char *filename, const char *encoding)
         perror("malloc");
 	return(NULL);
     }
+    memset(ctxt, 0, sizeof(htmlParserCtxt));
     htmlInitParserCtxt(ctxt);
     inputStream = (htmlParserInputPtr) xmlMalloc(sizeof(htmlParserInput));
     if (inputStream == NULL) {
@@ -2654,11 +2757,13 @@ htmlCreateFileParserCtxt(const char *filename, const char *encoding)
 	xmlFree(ctxt);
 	return(NULL);
     }
+    memset(inputStream, 0, sizeof(htmlParserInput));
 
     inputStream->filename = xmlMemStrdup(filename);
     inputStream->line = 1;
     inputStream->col = 1;
     inputStream->buf = buf;
+    inputStream->directory = NULL;
 
     inputStream->base = inputStream->buf->buffer->content;
     inputStream->cur = inputStream->buf->buffer->content;
