@@ -552,7 +552,7 @@ attribute(void *ctx, const xmlChar *fullname, const xmlChar *value)
     ret = xmlNewNsProp(ctxt->node, namespace, name, NULL);
 
     if (ret != NULL) {
-        if (ctxt->replaceEntities == 0)
+        if ((ctxt->replaceEntities == 0) && (!ctxt->html))
 	    ret->val = xmlStringGetNodeList(ctxt->myDoc, value);
 	else
 	    ret->val = xmlNewDocText(ctxt->myDoc, value);
@@ -811,6 +811,12 @@ characters(void *ctx, const xmlChar *ch, int len)
      * concatenate it, else create a new node of type text.
      */
 
+    if (ctxt->node == NULL) {
+#ifdef DEBUG_SAX_TREE
+	fprintf(stderr, "add chars: ctxt->node == NULL !\n");
+#endif
+        return;
+    }
     lastChild = xmlGetLastChild(ctxt->node);
 #ifdef DEBUG_SAX_TREE
     fprintf(stderr, "add chars to %s \n", ctxt->node->name);
