@@ -78,7 +78,7 @@ typedef xmlParserNodeInfoSeq *xmlParserNodeInfoSeqPtr;
  * The parser is not a state based parser, but we need to maintain
  * minimum state informations, especially for entities processing.
  */
-typedef enum xmlParserInputState {
+typedef enum {
     XML_PARSER_EOF = 0,
     XML_PARSER_PROLOG,
     XML_PARSER_CONTENT,
@@ -141,6 +141,13 @@ typedef struct _xmlParserCtxt {
     int                 token;        /* next char look-ahead */    
 
     char           *directory;        /* the data directory */
+
+    /* Node name stack only used for HTML parsing */
+    xmlChar           *name;          /* Current parsed Node */
+    int                nameNr;        /* Depth of the parsing stack */
+    int                nameMax;       /* Max depth of the parsing stack */
+    xmlChar *         *nameTab;       /* array of nodes */
+
 } _xmlParserCtxt;
 typedef _xmlParserCtxt xmlParserCtxt;
 typedef xmlParserCtxt *xmlParserCtxtPtr;
@@ -272,10 +279,10 @@ int		xmlParserInputGrow	(xmlParserInputPtr in,
 /**
  * xmlChar handling
  */
-xmlChar *		xmlStrdup		(const xmlChar *cur);
-xmlChar *		xmlStrndup		(const xmlChar *cur,
+xmlChar *	xmlStrdup		(const xmlChar *cur);
+xmlChar *	xmlStrndup		(const xmlChar *cur,
 					 int len);
-xmlChar *		xmlStrsub		(const xmlChar *str,
+xmlChar *	xmlStrsub		(const xmlChar *str,
 					 int start,
 					 int len);
 const xmlChar *	xmlStrchr		(const xmlChar *str,
@@ -288,9 +295,9 @@ int		xmlStrncmp		(const xmlChar *str1,
 					 const xmlChar *str2,
 					 int len);
 int		xmlStrlen		(const xmlChar *str);
-xmlChar *		xmlStrcat		(xmlChar *cur,
+xmlChar *	xmlStrcat		(xmlChar *cur,
 					 const xmlChar *add);
-xmlChar *		xmlStrncat		(xmlChar *cur,
+xmlChar *	xmlStrncat		(xmlChar *cur,
 					 const xmlChar *add,
 					 int len);
 
@@ -318,6 +325,13 @@ int		xmlParseDocument	(xmlParserCtxtPtr ctxt);
 xmlDocPtr	xmlSAXParseDoc		(xmlSAXHandlerPtr sax,
 					 xmlChar *cur,
 					 int recovery);
+int		xmlSAXUserParseFile	(xmlSAXHandlerPtr sax,
+					 void *user_data,
+					 const char *filename);
+int		xmlSAXUserParseMemory	(xmlSAXHandlerPtr sax,
+					 void *user_data,
+					 char *buffer,
+					 int size);
 xmlDocPtr	xmlSAXParseMemory	(xmlSAXHandlerPtr sax,
 					 char *buffer,
                                    	 int size,
