@@ -684,7 +684,7 @@ xmlNodeListGetString(xmlDocPtr doc, xmlNodePtr list, int inLine) {
 
     while (node != NULL) {
         if (node->type == XML_TEXT_NODE) {
-	    if (inLine)
+	    if ((inLine) || (doc->type == XML_HTML_DOCUMENT_NODE))
 		ret = xmlStrcat(ret, node->content);
 	    else {
 	        xmlChar *buffer;
@@ -776,6 +776,10 @@ xmlNewProp(xmlNodePtr node, const xmlChar *name, const xmlChar *value) {
 	    prev->next = cur;
 	}
     }
+#ifndef XML_WITHOUT_CORBA
+    cur->_private = NULL;
+    cur->vepv = NULL;
+#endif    
     return(cur);
 }
 
@@ -1078,6 +1082,10 @@ xmlNewText(const xmlChar *content) {
 	cur->content = xmlStrdup(content);
     else 
 	cur->content = NULL;
+#ifndef XML_WITHOUT_CORBA
+    cur->_private = NULL;
+    cur->vepv = NULL;
+#endif    
     return(cur);
 }
 
@@ -1129,6 +1137,10 @@ xmlNewReference(xmlDocPtr doc, const xmlChar *name) {
 	cur->content = ent->content;
     else
         cur->content = NULL;
+#ifndef XML_WITHOUT_CORBA
+    cur->_private = NULL;
+    cur->vepv = NULL;
+#endif    
     return(cur);
 }
 
@@ -1186,6 +1198,10 @@ xmlNewTextLen(const xmlChar *content, int len) {
 	cur->content = xmlStrndup(content, len);
     else 
 	cur->content = NULL;
+#ifndef XML_WITHOUT_CORBA
+    cur->_private = NULL;
+    cur->vepv = NULL;
+#endif    
     return(cur);
 }
 
@@ -1244,6 +1260,10 @@ xmlNewComment(const xmlChar *content) {
 	cur->content = xmlStrdup(content);
     else 
 	cur->content = NULL;
+#ifndef XML_WITHOUT_CORBA
+    cur->_private = NULL;
+    cur->vepv = NULL;
+#endif    
     return(cur);
 }
 
@@ -1284,6 +1304,10 @@ xmlNewCDataBlock(xmlDocPtr doc, const xmlChar *content, int len) {
 	cur->content = xmlStrndup(content, len);
     } else 
 	cur->content = NULL;
+#ifndef XML_WITHOUT_CORBA
+    cur->_private = NULL;
+    cur->vepv = NULL;
+#endif    
     return(cur);
 }
 
@@ -1925,8 +1949,6 @@ xmlCopyDoc(xmlDocPtr doc, int recursive) {
  *
  * Searches the language of a node, i.e. the values of the xml:lang
  * attribute or the one carried by the nearest ancestor.
- *
- * Returns a pointer to the lang value, or NULL if not found
  */
 void
 xmlNodeSetLang(xmlNodePtr cur, const xmlChar *lang) {
@@ -1991,6 +2013,7 @@ xmlNodeGetContent(xmlNodePtr cur) {
         case XML_ENTITY_NODE:
         case XML_COMMENT_NODE:
         case XML_DOCUMENT_NODE:
+        case XML_HTML_DOCUMENT_NODE:
         case XML_DOCUMENT_TYPE_NODE:
         case XML_NOTATION_NODE:
 	    return(NULL);
@@ -2044,6 +2067,7 @@ xmlNodeSetContent(xmlNodePtr cur, const xmlChar *content) {
 		cur->content = NULL;
 	    break;
         case XML_DOCUMENT_NODE:
+        case XML_HTML_DOCUMENT_NODE:
         case XML_DOCUMENT_TYPE_NODE:
 	    break;
         case XML_NOTATION_NODE:
@@ -2093,6 +2117,7 @@ xmlNodeSetContentLen(xmlNodePtr cur, const xmlChar *content, int len) {
 		cur->content = NULL;
 	    break;
         case XML_DOCUMENT_NODE:
+        case XML_HTML_DOCUMENT_NODE:
         case XML_DOCUMENT_TYPE_NODE:
 	    break;
         case XML_NOTATION_NODE:
@@ -2158,6 +2183,7 @@ xmlNodeAddContentLen(xmlNodePtr cur, const xmlChar *content, int len) {
 	    if (content != NULL)
 		cur->content = xmlStrncat(cur->content, content, len);
         case XML_DOCUMENT_NODE:
+        case XML_HTML_DOCUMENT_NODE:
         case XML_DOCUMENT_TYPE_NODE:
 	    break;
         case XML_NOTATION_NODE:
@@ -2640,6 +2666,20 @@ xmlBufferCCat(xmlBufferPtr buf, const char *str) {
     }
 }
 
+/**
+ * xmlBufferLastChar:
+ * @buf:  the buffer to dump
+ *
+ * Get the last char of the buffer
+ *
+ * Returns the last char from the buffer or 0 if empty
+ */
+xmlChar
+xmlBufferLastChar(xmlBufferPtr buf) {
+    if ((buf == NULL) || (buf->use <= 0)) return(0);
+    return(buf->content[buf->use - 1]);
+}
+
 /**
  * xmlBufferWriteCHAR:
  * @buf:  the XML buffer
@@ -2877,38 +2917,38 @@ xmlAttrListDump(xmlBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur) {
 
 
 static void
-xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level);
+xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level,
+            int format);
 /**
  * xmlNodeListDump:
  * @buf:  the XML buffer output
  * @doc:  the document
  * @cur:  the first node
  * @level: the imbrication level for indenting
+ * @format: is formatting allowed
  *
  * Dump an XML node list, recursive behaviour,children are printed too.
  */
 static void
-xmlNodeListDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level) {
-    int needIndent = 0, i;
+xmlNodeListDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level,
+                int format) {
+    int i;
 
     if (cur == NULL) {
         fprintf(stderr, "xmlNodeListDump : node == NULL\n");
 	return;
     }
     while (cur != NULL) {
-        if ((cur->type != XML_TEXT_NODE) &&
-	    (cur->type != XML_ENTITY_REF_NODE)) {
-	    if (!needIndent) {
-	        needIndent = 1;
-		xmlBufferWriteChar(buf, "\n");
-	    }
+	if ((format) && (xmlIndentTreeOutput) &&
+	    (cur->type == XML_ELEMENT_NODE))
+	    for (i = 0;i < level;i++)
+		xmlBufferWriteChar(buf, "  ");
+        xmlNodeDump(buf, doc, cur, level, format);
+	if (format) {
+	    xmlBufferWriteChar(buf, "\n");
 	}
-        xmlNodeDump(buf, doc, cur, level);
 	cur = cur->next;
     }
-    if ((xmlIndentTreeOutput) && (needIndent))
-	for (i = 1;i < level;i++)
-	    xmlBufferWriteChar(buf, "  ");
 }
 
 /**
@@ -2917,12 +2957,15 @@ xmlNodeListDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level) {
  * @doc:  the document
  * @cur:  the current node
  * @level: the imbrication level for indenting
+ * @format: is formatting allowed
  *
  * Dump an XML node, recursive behaviour,children are printed too.
  */
 static void
-xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level) {
+xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level,
+            int format) {
     int i;
+    xmlNodePtr tmp;
 
     if (cur == NULL) {
         fprintf(stderr, "xmlNodeDump : node == NULL\n");
@@ -2948,7 +2991,7 @@ xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level) {
 		xmlBufferWriteChar(buf, " ");
 		xmlBufferWriteCHAR(buf, cur->content);
 	    }
-	    xmlBufferWriteChar(buf, "?>\n");
+	    xmlBufferWriteChar(buf, "?>");
 	}
 	return;
     }
@@ -2956,7 +2999,7 @@ xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level) {
 	if (cur->content != NULL) {
 	    xmlBufferWriteChar(buf, "<!--");
 	    xmlBufferWriteCHAR(buf, cur->content);
-	    xmlBufferWriteChar(buf, "-->\n");
+	    xmlBufferWriteChar(buf, "-->");
 	}
 	return;
     }
@@ -2973,10 +3016,18 @@ xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level) {
         xmlBufferWriteChar(buf, "]]>");
 	return;
     }
-    if (xmlIndentTreeOutput)
-	for (i = 0;i < level;i++)
-	    xmlBufferWriteChar(buf, "  ");
 
+    if (format == 1) {
+    tmp = cur->childs;
+	while (tmp != NULL) {
+	    if ((tmp->type == XML_TEXT_NODE) || 
+		(tmp->type == XML_ENTITY_REF_NODE)) {
+		format = 0;
+		break;
+	    }
+	    tmp = tmp->next;
+	}
+    }
     xmlBufferWriteChar(buf, "<");
     if ((cur->ns != NULL) && (cur->ns->prefix != NULL)) {
         xmlBufferWriteCHAR(buf, cur->ns->prefix);
@@ -2990,7 +3041,7 @@ xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level) {
         xmlAttrListDump(buf, doc, cur->properties);
 
     if ((cur->content == NULL) && (cur->childs == NULL)) {
-        xmlBufferWriteChar(buf, "/>\n");
+        xmlBufferWriteChar(buf, "/>");
 	return;
     }
     xmlBufferWriteChar(buf, ">");
@@ -3004,7 +3055,11 @@ xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level) {
 	}
     }
     if (cur->childs != NULL) {
-	xmlNodeListDump(buf, doc, cur->childs, level + 1);
+	if (format) xmlBufferWriteChar(buf, "\n");
+	xmlNodeListDump(buf, doc, cur->childs, level + 1, format);
+	if ((xmlIndentTreeOutput) && (format))
+	    for (i = 0;i < level;i++)
+		xmlBufferWriteChar(buf, "  ");
     }
     xmlBufferWriteChar(buf, "</");
     if ((cur->ns != NULL) && (cur->ns->prefix != NULL)) {
@@ -3013,7 +3068,7 @@ xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level) {
     }
 
     xmlBufferWriteCHAR(buf, cur->name);
-    xmlBufferWriteChar(buf, ">\n");
+    xmlBufferWriteChar(buf, ">");
 }
 
 /**
@@ -3055,7 +3110,8 @@ xmlDocContentDump(xmlBufferPtr buf, xmlDocPtr cur) {
 	    xmlUpgradeOldNs(cur);
 	
 	while (child != NULL) {
-	    xmlNodeDump(buf, cur, child, 0);
+	    xmlNodeDump(buf, cur, child, 0, 1);
+	    xmlBufferWriteChar(buf, "\n");
 	    child = child->next;
 	}
     }
